@@ -7,8 +7,9 @@ package Padre::Util::SVN;
 use 5.008005;
 use strict;
 use warnings;
+use File::Spec ();
 
-our $VERSION = '0.76';
+our $VERSION = '0.90';
 
 my $PADRE = undef;
 
@@ -37,7 +38,7 @@ sub directory_revision {
 
 	# Find the headline revision
 	local $/ = undef;
-	open( my $fh, "<", $entries ) or return;
+	open( my $fh, '<', $entries ) or return;
 	my $buffer = <$fh>;
 	close $fh;
 
@@ -56,7 +57,7 @@ sub directory_revision {
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008-2010 The Padre development team as listed in Padre.pm.
+Copyright 2008-2011 The Padre development team as listed in Padre.pm.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
@@ -66,7 +67,7 @@ LICENSE file included with this module.
 
 =cut
 
-# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2011 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
