@@ -3,14 +3,16 @@ package Padre::Wx::FunctionList;
 use 5.008005;
 use strict;
 use warnings;
+use Carp                  ();
 use Scalar::Util          ();
 use Params::Util          ();
+use Padre::Feature        ();
 use Padre::Role::Task     ();
 use Padre::Wx::Role::View ();
 use Padre::Wx::Role::Main ();
 use Padre::Wx             ();
 
-our $VERSION = '0.76';
+our $VERSION = '0.90';
 our @ISA     = qw{
 	Padre::Role::Task
 	Padre::Wx::Role::View
@@ -73,36 +75,53 @@ sub new {
 	$self->SetSizerAndFit($sizerh);
 	$sizerh->SetSizeHints($self);
 
-	# Grab the kill focus to prevent deselection
-	Wx::Event::EVT_KILL_FOCUS(
-		$self->{list},
-		sub {
-			return;
-		},
-	);
-
-	# Double-click a function name
+	# Handle double-click on a function name
 	Wx::Event::EVT_LISTBOX_DCLICK(
 		$self,
 		$self->{list},
 		sub {
-			$self->on_list_item_activated( $_[1] );
+			my ( $this, $event ) = @_;
+			$self->on_list_item_activated($event);
+			return;
 		}
 	);
 
-	# Handle key events
+	# Handle double click on list.
+	# Overwrite to avoid stealing the focus back from the editor.
+	# On Windows this appears to kill the double-click feature entirely.
+	unless (Padre::Constant::WXWIN32) {
+		Wx::Event::EVT_LEFT_DCLICK(
+			$self->{list},
+			sub {
+				return;
+			}
+		);
+	}
+
+	# Handle key events in list
 	Wx::Event::EVT_KEY_UP(
 		$self->{list},
 		sub {
 			my ( $this, $event ) = @_;
-			if ( $event->GetKeyCode == Wx::WXK_RETURN ) {
+
+			my $code = $event->GetKeyCode;
+			if ( $code == Wx::WXK_RETURN ) {
 				$self->on_list_item_activated($event);
+			} elsif ( $code == Wx::WXK_ESCAPE ) {
+
+				# Escape key clears search and returns focus
+				# to the editor
+				$self->{search}->SetValue('');
+				my $editor = $self->current->editor;
+				$editor->SetFocus if $editor;
 			}
+
 			$event->Skip(1);
+			return;
 		}
 	);
 
-	# Handle key events
+	# Handle char events in search box
 	Wx::Event::EVT_CHAR(
 		$self->{search},
 		sub {
@@ -118,7 +137,6 @@ sub new {
 					$selection = 0;
 				}
 				$self->{list}->Select($selection);
-
 			} elsif ( $code == Wx::WXK_ESCAPE ) {
 
 				# Escape key clears search and returns focus
@@ -129,6 +147,7 @@ sub new {
 			}
 
 			$event->Skip(1);
+			return;
 		}
 	);
 
@@ -141,6 +160,10 @@ sub new {
 		}
 	);
 
+	if (Padre::Feature::STYLE_GUI) {
+		$self->recolour;
+	}
+
 	return $self;
 }
 
@@ -160,7 +183,11 @@ sub view_label {
 }
 
 sub view_close {
-	shift->main->show_functions(0);
+	$_[0]->main->show_functions(0);
+}
+
+sub view_stop {
+	$_[0]->task_reset;
 }
 
 
@@ -171,32 +198,16 @@ sub view_close {
 # Event Handlers
 
 sub on_list_item_activated {
-	my $self  = shift;
-	my $event = shift;
+	my $self   = shift;
+	my $event  = shift;
+	my $editor = $self->current->editor or return;
 
 	# Which sub did they click
-	my $subname = $self->{list}->GetStringSelection;
-	unless ( defined Params::Util::_STRING($subname) ) {
-		return;
+	my $name = $self->{list}->GetStringSelection;
+	if ( defined Params::Util::_STRING($name) ) {
+		$editor->goto_function($name);
 	}
 
-	# Locate the function
-	my $document = $self->current->document or return;
-	my $editor = $document->editor;
-	my ( $start, $end ) = Padre::Util::get_matches(
-		$editor->GetText,
-		$document->get_function_regex($subname),
-		$editor->GetSelection, # Provides two params
-	);
-	unless ( defined $start ) {
-
-		# Couldn't find it
-		return;
-	}
-
-	# Move the selection to the sub location
-	$editor->goto_pos_centerize($start);
-
 	return;
 }
 
@@ -216,12 +227,41 @@ sub gettext_label {
 	Wx::gettext('Functions');
 }
 
+# Pick up colouring from the current editor style
+sub recolour {
+	my $self   = shift;
+	my $config = $self->config;
+
+	# Load the editor style
+	require Padre::Wx::Editor;
+	my $data = Padre::Wx::Editor::data( $config->editor_style ) or return;
+
+	# Find the colours we need
+	my $foreground = $data->{padre}->{colors}->{PADRE_BLACK}->{foreground};
+	my $background = $data->{padre}->{background};
+
+	# Apply them to the widgets
+	if ( defined $foreground and defined $background ) {
+		$foreground = Padre::Wx::color($foreground);
+		$background = Padre::Wx::color($background);
+
+		$self->{list}->SetForegroundColour($foreground);
+		$self->{list}->SetBackgroundColour($background);
+
+		# $self->{search}->SetForegroundColour($foreground);
+		# $self->{search}->SetBackgroundColour($background);
+	}
+
+	return 1;
+}
+
 sub refresh {
 	my $self     = shift;
 	my $current  = shift or return;
 	my $document = $current->document;
 	my $search   = $self->{search};
 	my $list     = $self->{list};
+	my $lock     = $self->main->lock('UPDATE');
 
 	# Hide the widgets when no files are open
 	unless ($document) {
@@ -245,19 +285,18 @@ sub refresh {
 	}
 
 	# Nothing to do if there is no content
-	if ( $document->is_unused ) {
-		return 1;
+	my $task = $document->task_functions;
+	if ( $document->is_unused or not $task ) {
+		$list->Clear;
+		return;
 	}
 
 	# Launch the background task
-	my $task = $document->task_functions or return;
 	$self->task_request(
 		task  => $task,
 		text  => $document->text_get,
 		order => $current->config->main_functions_order,
 	);
-
-	return 1;
 }
 
 # Set an updated method list from the task
@@ -302,7 +341,7 @@ sub render {
 
 1;
 
-# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2011 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
